@@ -22,10 +22,14 @@ from abst.wrappers import mark_on_exit
 class Bastion:
     stopped = False
     session_list = []
+    session_desc = dict()
+    custom_ssh_options: str = "-o ServerAliveInterval=20"
 
-    def __init__(self, context_name=None):
+    def __init__(self, context_name=None, region=None):
         self.context_name = context_name
+        self.region = region
         self.shell: bool = False
+        self.tries: int = 10
         self.connected: bool = False
         self.active_tunnel: subprocess.Popen = Optional[None]
         self.response: Optional[dict] = None
@@ -42,6 +46,9 @@ class Bastion:
     def get_bastion_state(self) -> dict:
         session_id = self.response["id"]
         config = oci.config.from_file()
+        if self.region:
+            config["region"] = self.region
+
         req = oci.bastion.BastionClient(config).get_session(session_id)
 
         try:
@@ -59,18 +66,21 @@ class Bastion:
             self.active_tunnel.send_signal(signal.SIGTERM)
             sess_id = self.response["id"]
             Bastion.session_list.remove(sess_id)
+            region = Bastion.session_desc.pop(sess_id)
             print(f"Cleaning {self.get_print_name()}")
-            self.delete_bastion_session(sess_id)
+            self.delete_bastion_session(sess_id, region)
             print(f"Removed Session {self.get_print_name()}")
         except Exception:
             print(f"Looks like Bastion is already deleted {self.get_print_name()}")
 
     @classmethod
-    def delete_bastion_session(cls, sess_id):
+    def delete_bastion_session(cls, sess_id, region=None):
         print("Removing Bastion session")
 
         try:
             config = oci.config.from_file()
+            if region:
+                config["region"] = region
             while True:
                 try:
                     if oci.bastion.BastionClient(config).get_session(
@@ -96,7 +106,6 @@ class Bastion:
             logging.debug(f"Exception {ex}")
             rich.print(f"[red]Invalid Config in abst {self.get_print_name()}[/red]")
             exit(1)
-            return  # Just for Pycharm
 
         bid, response = self.load_response(res)
 
@@ -110,13 +119,13 @@ class Bastion:
             self.current_status = "creating bastion session succeeded"
             rich.print(
                 f"Created Session with id '{bid}' on Bastion {self.get_print_name()} "
-                f"'{response['data']['bastion-name']}'")
+                f"'{response['bastion_name']}'")
 
         rich.print("Auto resolving target details from response")
         host = creds["host"]
-        target_details = response["target-resource-details"]
-        ip = target_details["target-resource-private-ip-address"]
-        port = target_details["target-resource-port"]
+        target_details = response["target_resource_details"]
+        ip = target_details["target_resource_private_ip_address"]
+        port = target_details["target_resource_port"]
 
         self.current_status = "waiting for session init"
         self.wait_for_prepared()
@@ -124,12 +133,18 @@ class Bastion:
         sleep(1)  # Precaution init delay
 
         self.current_status = "digging tunnel"
-        ssh_tunnel_args = self.run_ssh_tunnel_managed_session(bid, host,
-                                                              creds["private-key-path"],
-                                                              creds[
-                                                                  "resource-os-username"],
-                                                              ip, port,
-                                                              shell)
+        ssh_tunnel_args, exit_code = self.run_ssh_tunnel_managed_session(bid, host,
+                                                                         creds["private-key-path"],
+                                                                         creds[
+                                                                             "resource-os-username"],
+                                                                         ip, port,
+                                                                         shell)
+        logging.info(f"SSH tunnel exited with code {exit_code}")
+        if exit_code == 0:
+            print(f"User requested termination {self.get_print_name()}")
+            self.current_status = "terminated by user"
+            self.kill()
+            return
 
         while status := (sdata := self.get_bastion_state())[
                             "lifecycle_state"] == "ACTIVE":
@@ -207,14 +222,16 @@ class Bastion:
         print(f"Bastion {self.get_print_name()} initialized")
         print(f"Initializing SSH Tunnel for {self.get_print_name()}")
 
-        ssh_tunnel_args = f'ssh -i {private_key_path} -o ProxyCommand="ssh -i {private_key_path} -W %h:%p -p {port} {bid}@{host} -A" -p {port} {username}@{ip} -A'
-        self.__run_ssh_tunnel_call(ssh_tunnel_args, shell, already_split=True)
-        return ssh_tunnel_args
+        ssh_tunnel_args = (f'ssh -i {private_key_path} {self.custom_ssh_options} '
+                           f'-o ProxyCommand="ssh -i {private_key_path} -W %h:%p -p {port} {bid}@{host} -A" -p {port} '
+                           f'{username}@{ip} -A')
+        exit_code = self.__run_ssh_tunnel_call(ssh_tunnel_args, shell, already_split=True)
+        return ssh_tunnel_args, exit_code
 
     def run_ssh_tunnel_port_forward(self, bid, host, ip, port, shell, local_port, ssh_pub_key_path):
         print(f"Bastion {self.get_print_name()} initialized")
         print(f"Initializing SSH Tunnel for {self.get_print_name()}")
-        ssh_tunnel_arg_str = f"ssh -o ServerAliveInterval=20 -N -L {local_port}:{ip}:{port} -p 22 {bid}@{host} -vvv -i {ssh_pub_key_path.strip('.pub')}"
+        ssh_tunnel_arg_str = f"ssh {self.custom_ssh_options} -N -L {local_port}:{ip}:{port} -p 22 {bid}@{host} -vvv -i {ssh_pub_key_path.strip('.pub')}"
         self.__run_ssh_tunnel(ssh_tunnel_arg_str, shell)
         return ssh_tunnel_arg_str
 
@@ -241,16 +258,25 @@ class Bastion:
     def create_bastion_forward_port_session(self, creds):
         ssh_key_path = self.get_ssh_pub_key_path(creds)
 
+        if "region" not in creds.keys():
+            rich.print("Missing region, will use profile default")
+            rich.print("If you want to add region, run abst config upgrade <ctx-name>")
+
+        if not Path(ssh_key_path).exists():
+            rich.print("[red]SSH key has invalid path[/red]")
+            exit(1)
+
         res = self.__create_bastion_session_port_forward(creds["bastion-id"],
                                                          creds["target-ip"],
                                                          f'{creds["default-name"]}-ctx-'
                                                          f'{self.get_print_name()}',
                                                          int(creds["target-port"]),
                                                          ssh_key_path,
-                                                         int(creds["ttl"]), False)
+                                                         int(creds["ttl"]), False, creds.get("region", None))
         try:
             trs = Bastion.parse_response(res)
             Bastion.session_list.append(trs["id"])
+            Bastion.session_desc[trs["id"]] = creds.get("region", None)
             logging.debug(f"Added session id of {self.context_name}")
         except:
             pass
@@ -258,6 +284,15 @@ class Bastion:
 
     def create_bastion_ssh_session_managed(self, creds):
         ssh_key_path = self.get_ssh_pub_key_path(creds)
+
+        if "region" not in creds.keys():
+            rich.print("Missing region, will use profile default")
+            rich.print("If you want to add region, run abst config upgrade <ctx-name>")
+
+        if not Path(ssh_key_path).exists():
+            rich.print("[red]SSH key has invalid path[/red]")
+            exit(1)
+
         try:
             res = self.__create_bastion_ssh_session_managed(creds["bastion-id"],
                                                             creds["resource-id"],
@@ -265,11 +300,12 @@ class Bastion:
                                                             f'{self.get_print_name()}',
                                                             creds["resource-os-username"],
                                                             ssh_key_path,
-                                                            int(creds["ttl"]), False
+                                                            int(creds["ttl"]), False, creds.get("region", None)
                                                             )
             try:
                 trs = Bastion.parse_response(res)
                 Bastion.session_list.append(trs["id"])
+                Bastion.session_desc[trs["id"]] = creds.get("region", None)
                 logging.debug(f"Added session id of {self.context_name}")
             except:
                 pass
@@ -278,6 +314,15 @@ class Bastion:
             rich.print(f"Missing filled out parameter for '{self.get_print_name()}' {ex}")
             exit(1)
 
+    @classmethod
+    def get_contexts(cls) -> dict:
+        out = dict()
+        for context in default_contexts_location.iterdir():
+            with open(context, "r") as f:
+                out[context.name.replace(".json", "")] = json.load(f)
+
+        return out
+
     @classmethod
     def get_creds_path_resolve(cls, context_name) -> Path:
         if context_name is None or context_name == "default":
@@ -378,7 +423,7 @@ class Bastion:
 
     @classmethod
     def __create_bastion_session_port_forward(cls, bastion_id, ip, name, port: int, ssh_path,
-                                              ttl: int, shell):
+                                              ttl: int, shell, region: Optional[str]):
         public_key = get_public_key(ssh_path)
         sess_details = oci.bastion.models.CreateSessionDetails(bastion_id=bastion_id,
                                                                target_resource_details=oci.bastion.models.CreatePortForwardingSessionTargetResourceDetails(
@@ -391,6 +436,10 @@ class Bastion:
                                                                key_type="PUB",
                                                                session_ttl_in_seconds=ttl)
         config = oci.config.from_file()
+        # Overriding region from user profile
+        if region:
+            config["region"] = region
+
         req = oci.bastion.BastionClient(config).create_session(sess_details)
 
         print("Creating Port Forward Session")
@@ -400,7 +449,7 @@ class Bastion:
 
     @classmethod
     def __create_bastion_ssh_session_managed(cls, bastion_id, resource_id, name,
-                                             os_username, ssh_path, ttl, shell):
+                                             os_username, ssh_path, ttl, shell, region: Optional[str]):
         if Bastion.stopped:
             return
         public_key = get_public_key(ssh_path)
@@ -408,13 +457,17 @@ class Bastion:
                                                                target_resource_details=oci.bastion.models.CreateManagedSshSessionTargetResourceDetails(
                                                                    session_type="MANAGED_SSH",
                                                                    target_resource_id=resource_id,
-                                                                   target_os_username=os_username),
+                                                                   target_resource_operating_system_user_name=os_username),
                                                                key_details=oci.bastion.models.PublicKeyDetails(
                                                                    public_key_content=public_key),
                                                                display_name=name,
                                                                key_type="PUB",
                                                                session_ttl_in_seconds=ttl)
         config = oci.config.from_file()
+
+        if region:
+            config["region"] = region
+
         req = oci.bastion.BastionClient(config).create_session(sess_details)
 
         print("Creating Managed SSH Session")
@@ -462,7 +515,7 @@ class Bastion:
 
     def __run_ssh_tunnel_call(self, ssh_tunnel_arg_str, shell, already_split=False):
 
-        os.system(ssh_tunnel_arg_str)
+        return os.system(ssh_tunnel_arg_str)
 
     def __run_ssh_tunnel(self, ssh_tunnel_arg_str, shell, already_split=False):
         """
@@ -473,12 +526,17 @@ class Bastion:
         :return:
         """
         args_split = self.process_args(already_split, shell, ssh_tunnel_arg_str)
+        try:
+            p = subprocess.Popen(args_split, stdout=subprocess.PIPE,
+                                 stderr=subprocess.STDOUT, shell=shell)
+        except FileNotFoundError:
+            self.connected = False
+            rich.print("Failed to establish SSH tunnel due to different setting os shell terminal, [red]try to run "
+                       "again with --shell[/red]")
+            return True
 
-        p = subprocess.Popen(args_split, stdout=subprocess.PIPE,
-                             stderr=subprocess.STDOUT, shell=shell)
         self.active_tunnel = p
-        tries = 3
-        while p.poll() is None and tries >= 0:
+        while p.poll() is None and self.tries >= 0:
             line = p.stdout.readline().decode("utf-8").strip()
             line_err = None
 
@@ -500,7 +558,7 @@ class Bastion:
                     f"{datetime.datetime.now()}")
                 self.connected = True
                 self.current_status = "connected"
-                tries = 3
+                self.tries = 10
 
             if not line and not line_err:
                 sleep(0.1)
@@ -518,12 +576,16 @@ class Bastion:
             rich.print(
                 f"({self.get_print_name()}) "
                 f"Please check you configuration, for more info use --debug flag")
-            if tries <= 0:
+            if self.tries <= 5:
+                rich.print("[red]If this continues to happen without connection it can be because ip changed in ["
+                           "yellow]~/.ssh/known_hosts[/yellow], delete it and"
+                           " try again[/red]")
+            if self.tries <= 0:
                 self.kill()
                 self.current_status = "Failed"
                 exit(255)
             else:
-                tries -= 1
-                self.current_status = f"failed {tries} left"
+                self.tries -= 1
+                self.current_status = f"failed {self.tries} left"
         self.connected = False
         return True
