@@ -1,22 +1,25 @@
+import json
 import logging
 import os
 import re
 import signal
 import subprocess
+from pathlib import Path
 from time import sleep
 
 import click
 import rich
 from InquirerPy import inquirer
+from requests import ConnectTimeout
 from rich.logging import RichHandler
 
 from abst.__version__ import __version_name__, __version__
 from abst.bastion_support.bastion_scheduler import BastionScheduler
+from abst.bastion_support.oci_bastion import Bastion
 from abst.cfg_func import __upgrade
 from abst.config import default_creds_path, default_contexts_location, default_conf_path
 from abst.dialogs import helm_login_dialog
 from abst.notifier.version_notifier import Notifier
-from abst.bastion_support.oci_bastion import Bastion
 from abst.tools import get_context_path, display_scheduled
 
 
@@ -42,6 +45,30 @@ def parallel():
     pass
 
 
+@cli.group(help="Contexts commands")
+def context():
+    pass
+
+
+@context.command("list", help="Will list all contexts in ~/.abst/context/ folder")
+def _list():
+    rich.print("Contexts:")
+    for file in Path(default_contexts_location).iterdir():
+        rich.print(f"    {file.name.replace('.json', '')}")
+
+
+@context.command(help="Will display JSON format of context")
+@click.argument("name")
+def display(name):
+    if name in [file.name.replace(".json", "") for file in
+                Path(default_contexts_location).iterdir()]:
+        rich.print("[bold]Context config contents:[/bold]\n")
+        with open(Path(default_contexts_location) / (name + ".json"), "r") as f:
+            rich.print_json(data=json.load(f))
+    else:
+        rich.print("[red]Context does not exists[/red]")
+
+
 @parallel.command("add", help="Add Bastion to stack")
 @click.option("--debug", is_flag=True, default=False)
 @click.argument("context-name", default="default")
@@ -71,19 +98,21 @@ def remove(debug, context_name):
 
 @parallel.command("run", help="Run All Bastions in fullauto")
 @click.option("--debug", is_flag=True, default=False)
-def run(debug):
+@click.option("-y", is_flag=True, default=False)
+def run(debug, y):
     setup_calls(debug)
     display_scheduled()
-    try:
-        confirm = inquirer.confirm(
-            "Do you really want to run following contexts?"
-        ).execute()
-    except KeyError:
-        rich.print("Unknown inquirer error, continuing...")
-        confirm = True
-    if not confirm:
-        rich.print("[green]Cancelling, nothing started[/green]")
-        exit(0)
+    if not y:
+        try:
+            confirm = inquirer.confirm(
+                "Do you really want to run following contexts?"
+            ).execute()
+        except KeyError:
+            rich.print("Unknown inquirer error, continuing...")
+            confirm = True
+        if not confirm:
+            rich.print("[green]Cancelling, nothing started[/green]")
+            exit(0)
     BastionScheduler.run()
 
 
@@ -98,8 +127,8 @@ def display(debug):
 @cli.command(
     "use",
     help="Will Switch context to be used,"
-    " use default for default context specified"
-    " in creds.json",
+         " use default for default context specified"
+         " in creds.json",
 )
 @click.option("--debug", is_flag=True, default=False)
 @click.argument("context-name", default="")
@@ -253,7 +282,10 @@ def create():
 
 def setup_calls(debug):
     setup_debug(debug)
-    Notifier.notify()
+    try:
+        Notifier.notify()
+    except (ConnectionError, ConnectTimeout):
+        return False
 
 
 def setup_debug(debug):
@@ -269,42 +301,9 @@ def setup_debug(debug):
     )
 
 
-@create.group(help="Group of commands for Creating Port Forward Sessions")
-def forward():
-    pass
-
-
-@create.group(help="Group of commands for Creating Managed SSH Sessions")
-def managed():
-    pass
-
-
-@forward.command(
-    "single",
-    help="Creates only one bastion session and keeps reconnecting until"
-    " its deleted, does not create any more Bastion sessions",
-)
-@click.option("--shell", is_flag=True, default=False)
-@click.option("--debug", is_flag=True, default=False)
-@click.argument("context-name", default=None, required=False)
-def single_forward(shell, debug, context_name):
-    """Creates only one bastion session
-    ,connects and reconnects until its ttl runs out"""
-    setup_calls(debug)
-
-    if context_name is None:
-        conf = Bastion.load_config()
-        used_name = conf["used_context"]
-    else:
-        used_name = context_name
-
-    Bastion(used_name).create_forward_loop(shell=shell)
-
-
-@forward.command(
-    "fullauto",
-    help="Creates and connects to Bastion session indefinitely until "
-    "terminated by user",
+@create.command(
+    "forward",
+    help="Creates and connects to Bastion session indefinitely until terminated by user",
 )
 @click.option("--shell", is_flag=True, default=False)
 @click.option("--debug", is_flag=True, default=False)
@@ -314,6 +313,11 @@ def fullauto_forward(shell, debug, context_name):
     automatically until terminated"""
 
     setup_calls(debug)
+
+    if context_name == "?":
+        print_eligible("required only for Port Forward session")
+        return
+
     if context_name is None:
         conf = Bastion.load_config()
         used_name = conf["used_context"]
@@ -321,36 +325,31 @@ def fullauto_forward(shell, debug, context_name):
         used_name = context_name
 
     while True:
-        Bastion(used_name).create_forward_loop(shell=shell)
+        Bastion(used_name, region=
+        Bastion.load_json(Bastion.get_creds_path_resolve(context_name)).get("region",
+                                                                            None)).create_forward_loop(
+            shell=shell)
 
         sleep(1)
 
 
-@managed.command(
-    "single",
-    help="Creates only one bastion session and keeps reconnecting until"
-    " its deleted, does not create any more Bastion sessions",
-)
-@click.option("--shell", is_flag=True, default=False)
-@click.option("--debug", is_flag=True, default=False)
-@click.argument("context-name", default=None, required=False)
-def single_managed(shell, debug, context_name):
-    """Creates only one bastion session
-    ,connects and reconnects until its ttl runs out"""
-    setup_calls(debug)
-    if context_name is None:
-        conf = Bastion.load_config()
-        used_name = conf["used_context"]
-    else:
-        used_name = context_name
-
-    Bastion(used_name).create_managed_loop(shell=shell)
+def print_eligible(searched: str):
+    contexts = Bastion.get_contexts()
+    rich.print("[bold]Configured contexts:[/bold]")
+    for key, context in contexts.items():
+        do_skip = False
+        for value in context.values():
+            if searched in value:
+                do_skip = True
+                break
+        if do_skip:
+            continue
+        rich.print(key)
 
 
-@managed.command(
-    "fullauto",
-    help="Creates and connects to Bastion session indefinitely until "
-    "terminated by user",
+@create.command(
+    "managed",
+    help="Creates and connects to Bastion session indefinitely until terminated by user",
 )
 @click.option("--shell", is_flag=True, default=False)
 @click.option("--debug", is_flag=True, default=False)
@@ -360,6 +359,10 @@ def fullauto_managed(shell, debug, context_name):
     automatically until terminated"""
     setup_calls(debug)
 
+    if context_name == "?":
+        print_eligible("required only for Managed SSH session")
+        return
+
     if context_name is None:
         conf = Bastion.load_config()
         used_name = conf["used_context"]
@@ -367,7 +370,10 @@ def fullauto_managed(shell, debug, context_name):
         used_name = context_name
 
     while True:
-        Bastion(used_name).create_managed_loop(shell=shell)
+        Bastion(used_name, region=
+        Bastion.load_json(Bastion.get_creds_path_resolve(context_name)).get("region",
+                                                                            None)).create_managed_loop(
+            shell=shell)
 
         sleep(1)
 
@@ -377,7 +383,7 @@ def fullauto_managed(shell, debug, context_name):
 @click.option("--debug", is_flag=True, default=False)
 def ssh_pod(pod_name, debug):
     setup_calls(debug)
-    found = list()
+
     try:
         rich.print("Fetching pods")
         pod_lines = (
@@ -389,9 +395,7 @@ def ssh_pod(pod_name, debug):
         rich.print("[red]kubectl not found on this machine[/red]")
         return
 
-    for pod_line in pod_lines:
-        if pod_name in pod_line:
-            found.append(pod_line)
+    found = list(filter(lambda pod_line: pod_name in pod_line, pod_lines))
 
     if len(found) > 1:
         data = re.sub(
@@ -416,6 +420,48 @@ def ssh_pod(pod_name, debug):
     )
 
 
+@cli.command("logs", help="Will get logs from a pod with containing string name")
+@click.argument("pod_name")
+@click.option("--debug", is_flag=True, default=False)
+def log_pod(pod_name, debug):
+    setup_calls(debug)
+
+    try:
+        rich.print("Fetching pods")
+        pod_lines = (
+            subprocess.check_output(f"kubectl get pods -A".split(" "))
+            .decode()
+            .split("\n")
+        )
+    except FileNotFoundError:
+        rich.print("[red]kubectl not found on this machine[/red]")
+        return
+
+    found = list(filter(lambda pod_line: pod_name in pod_line, pod_lines))
+
+    if len(found) > 1:
+        data = re.sub(
+            " +",
+            " ",
+            inquirer.select("Found more pods, choose one:", found).execute(),
+        ).split(" ")
+        pod_name_precise = data[1]
+    elif len(found) == 1:
+        tmp = found.pop()
+        data = re.sub(" +", " ", tmp).split(" ")
+        pod_name_precise = data[1]
+    else:
+        rich.print(f"[red]No pods with name {pod_name} found[/red]")
+        return
+
+    rich.print(
+        f"[green]Getting logs from {pod_name_precise} in namespace: {data[0]}[/green]"
+    )
+    os.system(
+        f"kubectl -n {data[0]} logs {pod_name_precise}"
+    )
+
+
 @cli.group("cp")
 def cp():
     pass
@@ -536,10 +582,10 @@ def helm_push(chart, debug):
 @click.argument("target_namespace")
 @click.option("--debug", is_flag=True, default=False)
 def cp_secret(
-    secret_name: str,
-    target_namespace: str,
-    source_namespace: str = "default",
-    debug=False,
+        secret_name: str,
+        target_namespace: str,
+        source_namespace: str = "default",
+        debug=False,
 ):
     """
     Copy Secret in current cluster from source namespace to target
